/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;
import org.w3c.dom.Document;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.minfo.BedDiffYearResult;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.jfree.StyledXYSeries;


public class BedDiffYearProcessor implements Processor, FacetTypes {

    private final static Logger logger =
            Logger.getLogger(BedDiffYearProcessor.class);

    protected static double GAP_TOLERANCE = 0.101d;

    @Override
    public void doOut(
            XYChartGenerator generator,
            ArtifactAndFacet aandf,
            Document theme,
            boolean visible,
            int axidx
    ) {
        CallContext context = generator.getCallContext();
        Object data = aandf.getData(context);
        if (data instanceof BedDiffYearResult) {
            String facetType = aandf.getFacetName();
            int index = 
               (BED_DIFFERENCE_YEAR_HEIGHT1.equals(facetType)
               || BED_DIFFERENCE_YEAR_HEIGHT1_FILTERED.equals(facetType))?0:1;
            doBedDifferenceHeightsOut(
               generator,
               (BedDiffYearResult) data,
               aandf, theme, visible, index, axidx);
            return;
        }
        logger.error("Can't process " + data.getClass().getName() + " objects");
    }

    @Override
    public boolean canHandle(String facetType) {
        return
               BED_DIFFERENCE_YEAR_HEIGHT1.equals(facetType)
            || BED_DIFFERENCE_YEAR_HEIGHT2.equals(facetType)
            || BED_DIFFERENCE_YEAR_HEIGHT1_FILTERED.equals(facetType)
            || BED_DIFFERENCE_YEAR_HEIGHT2_FILTERED.equals(facetType);
    }

    private void doBedDifferenceHeightsOut(
        XYChartGenerator generator,
        BedDiffYearResult data,
        ArtifactAndFacet bundle,
        Document attr,
        boolean visible,
        int idx,
        int axidx) {
         logger.debug("doBedDifferenceHeightsOut()");

        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), attr);
        double[][] points;
        if (idx == 0) {
            points = data.getHeights1Data();
        }
        else {
            points = data.getHeights2Data();
        }

        StyledSeriesBuilder.addPoints(series, points, false, GAP_TOLERANCE);

        generator.addAxisSeries(series, axidx, visible);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
